use crate::robot::connection_handler::ConnectionHandler;
use crate::robot::robot_manager::RobotManager;
use crate::robot::ConnectorManager;
use log::{error, info};
use prost::Message;
use raas_types::raas;
use raas_types::raas::cmd::Request;
use raas_types::raas::ping::Ping;
use raas_types::raas::{recv_raas_msg, send_raas_msg};
use std::sync::Arc;
use std::time::Duration;
use tokio::io::duplex;
use tokio::net::TcpListener;
use tokio::sync::Mutex;

mod robot;

#[tokio::main]
async fn main() {
    env_logger::init();
    info!("Starting...");

    let addr = "0.0.0.0:8080";

    let connector_manager = ConnectorManager {
        next_id: 0,
        connectors: Default::default(),
    };

    let connector_manager = Arc::new(Mutex::new(connector_manager));

    let tcp_listener = TcpListener::bind(addr).await.unwrap();

    let (mut main_stream, manager_stream) = duplex(10);

    let robot_manager = RobotManager::new(manager_stream);
    let connector_handler = ConnectionHandler::new(addr, tcp_listener);

    let robot_manager_connector_manager = connector_manager.clone();
    tokio::task::spawn(robot_manager.worker(robot_manager_connector_manager));
    tokio::task::spawn(connector_handler.worker(connector_manager));

    loop {
        info!("Sleeping");
        tokio::time::sleep(Duration::from_secs(5)).await;
        info!("Sending Ping");

        let ping = Ping {
            data: "ping msg".to_string(),
        };

        let req = Request {
            timestamp: 0,
            cmd: Some(raas::cmd::request::Cmd::RollCmd(raas::bot::roll::RollCmd {
                cmd: Some(raas::bot::roll::roll_cmd::Cmd::Ping(ping)),
            })),
        };

        let mut msg = Vec::new();

        req.encode(&mut msg).unwrap();

        send_raas_msg(&mut main_stream, msg).await.unwrap();

        info!("Wait for resp");
        let ret =
            tokio::time::timeout(Duration::from_secs(5), recv_raas_msg(&mut main_stream)).await;

        match ret {
            Ok(resp) => match resp {
                Ok(resp) => {
                    info!("Got response {:?}", resp);
                }
                Err(err) => {
                    error!("Error getting response: {}", err)
                }
            },
            Err(_) => {
                error!("Timed out waiting for response!")
            }
        }
    }
}
